@@ -3,7 +3,7 @@ package Module::Install::XSUtil;
 
 use 5.005_03;
 
-$VERSION = '0.20';
+$VERSION = '0.32';
 
 use Module::Install::Base;
 @ISA     = qw(Module::Install::Base);
@@ -46,24 +46,25 @@ sub _xs_initialize{
         $self->{xsu_initialized} = 1;
 
         if(!$self->cc_available()){
-            print "This package requires a C compiler, but it's not available.\n";
-            exit(0);
+            warn "This distribution requires a C compiler, but it's not available, stopped.\n";
+            exit;
         }
 
         $self->configure_requires(%ConfigureRequires);
         $self->build_requires(%BuildRequires);
         $self->requires(%Requires);
 
-        $self->makemaker_args(OBJECT => '$(O_FILES)');
+        $self->makemaker_args->{OBJECT} = '$(O_FILES)';
         $self->clean_files('$(O_FILES)');
+        $self->clean_files('*.stackdump') if $^O eq 'cygwin';
 
         if($self->_xs_debugging()){
             # override $Config{optimize}
             if(_is_msvc()){
-                $self->makemaker_args(OPTIMIZE => '-Zi');
+                $self->makemaker_args->{OPTIMIZE} = '-Zi';
             }
             else{
-                $self->makemaker_args(OPTIMIZE => '-g');
+                $self->makemaker_args->{OPTIMIZE} = '-g';
             }
             $self->cc_define('-DXS_ASSERT');
         }
@@ -85,30 +86,40 @@ sub _is_msvc{
     my $cc_available;
 
     sub cc_available {
-        return $cc_available if defined $cc_available;
+        return defined $cc_available ?
+            $cc_available :
+            ($cc_available = shift->can_cc())
+        ;
+    }
+
+    my $want_xs;
+    sub want_xs {
+        my $default = @_ ? shift : 1; # you're using this module, you /must/ want XS by default
+        return $want_xs if defined $want_xs;
 
         foreach my $arg(@ARGV){
             if($arg eq '--pp'){
-                return $cc_available = 0;
+                return $want_xs = 0;
             }
             elsif($arg eq '--xs'){
-                return $cc_available = 1;
+                return $want_xs = 1;
             }
         }
-
-        return $cc_available = shift->can_cc();
+        return $want_xs = $default;
     }
 }
 
 sub use_ppport{
     my($self, $dppp_version) = @_;
+    return if $self->{_ppport_ok}++;
 
     $self->_xs_initialize();
 
     my $filename = 'ppport.h';
 
-    $dppp_version ||= 0;
+    $dppp_version ||= 3.19; # the more, the better
     $self->configure_requires('Devel::PPPort' => $dppp_version);
+    $self->build_requires('Devel::PPPort' => $dppp_version);
 
     print "Writing $filename\n";
 
@@ -132,25 +143,56 @@ sub use_ppport{
     return;
 }
 
+sub use_xshelper {
+    my($self, $opt) = @_;
+    $self->_xs_initialize();
+    $self->use_ppport();
+
+    my $file = 'xshelper.h';
+    open my $fh, '>', $file or die "Cannot open $file for writing: $!";
+    print $fh $self->_xshelper_h();
+    close $fh or die "Cannot close $file: $!";
+    if(defined $opt) {
+        if($opt eq '-clean') {
+            $self->clean_files($file);
+        }
+        else {
+            $self->realclean_files($file);
+        }
+    }
+    return;
+}
+
+sub _gccversion {
+    my $res = `$Config{cc} --version`;
+    my ($version) = $res =~ /\(GCC\) ([0-9.]+)/;
+    no warnings 'numeric', 'uninitialized';
+    return sprintf '%g', $version;
+}
+
 sub cc_warnings{
     my($self) = @_;
 
     $self->_xs_initialize();
 
     if(_is_gcc()){
-        # Note: MSVC++ doesn't support C99, so -Wdeclaration-after-statement helps ensure C89 specs.
         $self->cc_append_to_ccflags(qw(-Wall));
 
-        no warnings 'numeric';
-        if($Config{gccversion} >= 4.00){
-            $self->cc_append_to_ccflags('-Wextra -Wdeclaration-after-statement -Wc++-compat');
+        my $gccversion = _gccversion();
+        if($gccversion >= 4.0){
+            # Note: MSVC++ doesn't support C99, so -Wdeclaration-after-statement helps
+            # ensure C89 specs.
+            $self->cc_append_to_ccflags(qw(-Wextra -Wdeclaration-after-statement));
+            if($gccversion >= 4.1) {
+                $self->cc_append_to_ccflags(qw(-Wc++-compat));
+            }
         }
         else{
-            $self->cc_append_to_ccflags('-W -Wno-comment');
+            $self->cc_append_to_ccflags(qw(-W -Wno-comment));
         }
     }
     elsif(_is_msvc()){
-        $self->cc_append_to_ccflags('-W3');
+        $self->cc_append_to_ccflags(qw(-W3));
     }
     else{
         # TODO: support other compilers
@@ -159,6 +201,46 @@ sub cc_warnings{
     return;
 }
 
+sub c99_available {
+    my($self) = @_;
+
+    return 0 if not $self->cc_available();
+
+    require File::Temp;
+    require File::Basename;
+
+    my $tmpfile = File::Temp->new(SUFFIX => '.c');
+
+    $tmpfile->print(<<'C99');
+inline // a C99 keyword with C99 style comments
+int test_c99() {
+    int i = 0;
+    i++;
+    int j = i - 1; // another C99 feature: declaration after statement
+    return j;
+}
+C99
+
+    $tmpfile->close();
+
+    system $Config{cc}, '-c', $tmpfile->filename;
+
+    (my $objname = File::Basename::basename($tmpfile->filename)) =~ s/\Q.c\E$/$Config{_o}/;
+    unlink $objname or warn "Cannot unlink $objname (ignored): $!";
+
+    return $? == 0;
+}
+
+sub requires_c99 {
+    my($self) = @_;
+    if(!$self->c99_available) {
+        warn "This distribution requires a C99 compiler, but $Config{cc} seems not to support C99, stopped.\n";
+        exit;
+    }
+    $self->_xs_initialize();
+
+    return;
+}
 
 sub cc_append_to_inc{
     my($self, @dirs) = @_;
@@ -168,7 +250,6 @@ sub cc_append_to_inc{
     for my $dir(@dirs){
         unless(-d $dir){
             warn("'$dir' not found: $!\n");
-            exit;
         }
 
         _verbose "inc: -I$dir" if _VERBOSE;
@@ -186,29 +267,70 @@ sub cc_append_to_inc{
     return;
 }
 
+sub cc_libs {
+    my ($self, @libs) = @_;
+
+    @libs = map{
+        my($name, $dir) = ref($_) eq 'ARRAY' ? @{$_} : ($_, undef);
+        my $lib;
+        if(defined $dir) {
+            $lib = ($dir =~ /^-/ ? qq{$dir } : qq{-L$dir });
+        }
+        else {
+            $lib = '';
+        }
+        $lib .= ($name =~ /^-/ ? qq{$name} : qq{-l$name});
+        _verbose "libs: $lib" if _VERBOSE;
+        $lib;
+    } @libs;
+
+    $self->cc_append_to_libs( @libs );
+}
+
 sub cc_append_to_libs{
     my($self, @libs) = @_;
 
     $self->_xs_initialize();
 
+    return unless @libs;
+
+    my $libs = join q{ }, @libs;
+
     my $mm = $self->makemaker_args;
 
-    my $libs = join q{ }, map{
-        my($name, $dir) = ref($_) eq 'ARRAY' ? @{$_} : ($_, undef);
-
-        $dir = qq{-L$dir } if defined $dir;
-        _verbose "libs: $dir-l$name" if _VERBOSE;
-        $dir . qq{-l$name};
-    } @libs;
-
-    if($mm->{LIBS}){
+    if ($mm->{LIBS}){
         $mm->{LIBS} .= q{ } . $libs;
     }
     else{
         $mm->{LIBS} = $libs;
     }
+    return $libs;
+}
 
-    return;
+sub cc_assert_lib {
+    my ($self, @dcl_args) = @_;
+
+    if ( ! $self->{xsu_loaded_checklib} ) {
+        my $loaded_lib = 0;
+        foreach my $checklib qw(inc::Devel::CheckLib Devel::CheckLib) {
+            eval "use $checklib 0.4";
+            if (!$@) {
+                $loaded_lib = 1;
+                last;
+            }
+        }
+
+        if (! $loaded_lib) {
+            warn "Devel::CheckLib not found in inc/ nor \@INC";
+            exit 0;
+        }
+
+        $self->{xsu_loaded_checklib}++;
+        $self->configure_requires( "Devel::CheckLib" => "0.4" );
+        $self->build_requires( "Devel::CheckLib" => "0.4" );
+    }
+
+    Devel::CheckLib::check_lib_or_exit(@dcl_args);
 }
 
 sub cc_append_to_ccflags{
@@ -287,7 +409,7 @@ sub requires_xs{
     $self->cc_append_to_inc (grep{ !$uniq{ $_ }++ } @inc);
 
     %uniq = ();
-    $self->cc_append_to_libs(grep{ !$uniq{ $_->[0] }++ } @libs);
+    $self->cc_libs(grep{ !$uniq{ $_->[0] }++ } @libs);
 
     return %added;
 }
@@ -327,6 +449,11 @@ sub cc_src_paths{
         push @{$C_ref}, $c unless grep{ $_ eq $c } @{$C_ref};
     }
 
+    $self->clean_files(map{
+        File::Spec->catfile($_, '*.gcov'),
+        File::Spec->catfile($_, '*.gcda'),
+        File::Spec->catfile($_, '*.gcno'),
+    } @dirs);
     $self->cc_append_to_inc('.');
 
     return;
@@ -386,7 +513,10 @@ sub install_headers{
         $ToInstall{$path} = File::Spec->join('$(INST_ARCHAUTODIR)', $ident);
 
         _verbose "install: $path as $ident" if _VERBOSE;
-        $self->_extract_functions_from_header_file($path);
+        my @funcs = $self->_extract_functions_from_header_file($path);
+        if(@funcs){
+            $self->cc_append_to_funclist(@funcs);
+        }
     }
 
     if(@not_found){
@@ -469,11 +599,7 @@ sub _extract_functions_from_header_file{
             }
     }
 
-    if(@functions){
-        $self->cc_append_to_funclist(@functions);
-    }
-
-    return;
+    return @functions;
 }
 
 
@@ -490,6 +616,113 @@ sub cc_append_to_funclist{
     return;
 }
 
+sub _xshelper_h {
+    my $h = <<'XSHELPER_H';
+:/* THIS FILE IS AUTOMATICALLY GENERATED BY Module::Install::XSUtil $VERSION. */
+:/*
+:=head1 NAME
+:
+:xshelper.h - Helper C header file for XS modules
+:
+:=head1 DESCRIPTION
+:
+:    // This includes all the perl header files and ppport.h
+:    #include "xshelper.h"
+:
+:=head1 SEE ALSO
+:
+:L<Module::Install::XSUtil>, where this file is distributed as a part of
+:
+:=head1 AUTHOR
+:
+:Fuji, Goro (gfx) E<lt>gfuji at cpan.orgE<gt>
+:
+:=head1 LISENCE
+:
+:Copyright (c) 2010, Fuji, Goro (gfx). All rights reserved.
+:
+:This library is free software; you can redistribute it and/or modify
+:it under the same terms as Perl itself.
+:
+:=cut
+:*/
+:
+:#ifdef __cplusplus
+:extern "C" {
+:#endif
+:
+:#define PERL_NO_GET_CONTEXT /* we want efficiency */
+:#include <EXTERN.h>
+:
+:#include <perl.h>
+:#define NO_XSLOCKS /* for exceptions */
+:#include <XSUB.h>
+:
+:#ifdef __cplusplus
+:} /* extern "C" */
+:#endif
+:
+:#include "ppport.h"
+:
+:/* portability stuff not supported by ppport.h yet */
+:
+:#ifndef STATIC_INLINE /* from 5.13.4 */
+:# if defined(__GNUC__) || defined(__cplusplus) || (defined(__STDC_VERSION__) && (__STDC_VERSION__ >= 199901L))
+:#   define STATIC_INLINE static inline
+:# else
+:#   define STATIC_INLINE static
+:# endif
+:#endif /* STATIC_INLINE */
+:
+:#ifndef __attribute__format__
+:#define __attribute__format__(a,b,c) /* nothing */
+:#endif
+:
+:#ifndef LIKELY /* they are just a compiler's hint */
+:#define LIKELY(x)   (x)
+:#define UNLIKELY(x) (x)
+:#endif
+:
+:#ifndef newSVpvs_share
+:#define newSVpvs_share(s) Perl_newSVpvn_share(aTHX_ STR_WITH_LEN(s), 0U)
+:#endif
+:
+:#ifndef get_cvs
+:#define get_cvs(name, flags) get_cv(name, flags)
+:#endif
+:
+:#ifndef GvNAME_get
+:#define GvNAME_get GvNAME
+:#endif
+:#ifndef GvNAMELEN_get
+:#define GvNAMELEN_get GvNAMELEN
+:#endif
+:
+:#ifndef CvGV_set
+:#define CvGV_set(cv, gv) (CvGV(cv) = (gv))
+:#endif
+:
+:/* general utility */
+:
+:#if PERL_BCDVERSION >= 0x5008005
+:#define LooksLikeNumber(x) looks_like_number(x)
+:#else
+:#define LooksLikeNumber(x) (SvPOKp(x) ? looks_like_number(x) : (I32)SvNIOKp(x))
+:#endif
+:
+:#define newAV_mortal() (AV*)sv_2mortal((SV*)newAV())
+:#define newHV_mortal() (HV*)sv_2mortal((SV*)newHV())
+:
+:#define DECL_BOOT(name) EXTERN_C XS(CAT2(boot_, name))
+:#define CALL_BOOT(name) STMT_START {            \
+:        PUSHMARK(SP);                           \
+:        CALL_FPTR(CAT2(boot_, name))(aTHX_ cv); \
+:    } STMT_END
+XSHELPER_H
+    $h =~ s/^://xmsg;
+    $h =~ s/\$VERSION\b/$Module::Install::XSUtil::VERSION/xms;
+    return $h;
+}
 
 package
     MY;
@@ -522,8 +755,7 @@ sub const_cccmd {
 
     return $cccmd
 }
-
 1;
 __END__
 
-#line 685
+#line 969
